/* 
 * enPathia
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib/gi18n.h>
#include <glib/gstdio.h>
#include <locale.h>
#include <string.h>
#include <sys/stat.h>
#include "calibration.h"
#include "clicker.h"
#include "enpathia.h"
#include "keyboard.h"
#include "menu.h"
#include "misc.h"
#include "protocol.h"
#include "profile.h"

Profile	current_profile;
gchar	current_profile_name [50];

gint	current_language;
gchar*	language_name [N_LANGUAGES] =
{ "en", "el", "es", "he", "de", "it", "pt", "fr" };

gint	current_keyb_layout;
gchar*	keyb_layout_name [N_KEYB_LAYOUTS] =
{ "en_US", "en_UK", "el", "es", "he", "de", "it", "pt", "fr" };
gchar*	keyb_layout_print_name [N_KEYB_LAYOUTS] =
{ "US", "UK", "EL", "ES", "HE", "DE", "IT", "PT", "FR" };
gchar*	keyb_dictionary_name [N_KEYB_LAYOUTS] =
{ "en", "en", "el", "es", "he", "de", "it", "pt", "fr" };

static void
set_default_values		(Profile* profile);
static gchar* 
path_from_name			(gchar* name);
static gint
save_profile			(gchar* name);
static void
get_system_language		(void);
static void
get_system_keyb_layout	(void);


	void
load_config_file (void)
{
	GKeyFile*	file;
	GError*		error;
	gchar*		value_str;
	gchar*		locale_dir;
	gchar*		config_file;
	gboolean	file_ok;
	gint		i;

	set_default_values (&current_profile);

	/* Open the config file */
	file = g_key_file_new ();

	config_file = get_config_file ("/config");

	error = NULL;
	file_ok = g_key_file_load_from_file (file, config_file,
			G_KEY_FILE_NONE, &error);

	g_free (config_file);

	if (file_ok)
	{
		/* Read the selected language */
		error = NULL;
		value_str = g_key_file_get_string (file, "language", "lang", &error);

		if (error == NULL && strcmp (value_str, ""))
		{
			for (i = 0; i < N_LANGUAGES; i++)
			{
				if (!strcmp (value_str, language_name [i]))
				{
					current_language = i;
					break;
				}
			}
		}
		else 
		{
			get_system_language ();
			file_ok = FALSE;
		}

		/* Read the selected keyboard layout */
		error = NULL;
		value_str = g_key_file_get_string (file, "keyboard", "layout", &error);

		if (error == NULL && strcmp (value_str, ""))
		{
			for (i = 0; i < N_KEYB_LAYOUTS; i++)
			{
				if (!strcmp (value_str, keyb_layout_name [i]))
				{
					current_keyb_layout = i;
					break;
				}
			}
		}
		else 
		{
			get_system_keyb_layout ();
			file_ok = FALSE;
		}


		/* Read the last used profile name */
		error = NULL;
		value_str = g_key_file_get_string (file, "profiles", "last", &error);

		/* And try to load it */
		if (load_profile (value_str) < 0)
			file_ok = FALSE;
	}
	else 
	{
		g_message ("Config file: %s", error->message);
		get_system_language ();
		get_system_keyb_layout ();
	}

	/* Gettext */
	locale_dir = get_data_file ("/locale/");
	if (!g_file_test (locale_dir, G_FILE_TEST_EXISTS))
	{
		g_free (locale_dir);
		locale_dir = g_strdup ("../po/locale/");
	}

	bindtextdomain ("enpathia", locale_dir);
	g_free (locale_dir);

	bind_textdomain_codeset ("enpathia", "UTF-8");
	textdomain ("enpathia");

	set_env_var ("LANG", language_name [current_language]);
	set_env_var ("LANGUAGE", language_name [current_language]);
	setlocale (LC_MESSAGES, language_name [current_language]);

	/* If there was an error, save a default profile */
	if (!file_ok)
	{
		strcpy (current_profile_name, _("User"));
		update_config_file ();
		save_current_profile ();
	}

	g_key_file_free (file);
}

	void
update_config_file (void)
{
	GKeyFile*	file;
	gchar*		data;
	gchar*		config_file;
	FILE*		f;

	/* Create the file */
	file = g_key_file_new ();

	/* Set the keys */
	g_key_file_set_string (file, "profiles", "last", current_profile_name);
	g_key_file_set_string (file, "language", "lang", language_name [current_language]);
	g_key_file_set_string (file, "keyboard", "layout", keyb_layout_name [current_keyb_layout]);

	/* And save it */
	data = g_key_file_to_data (file, NULL, NULL);

	config_file = get_config_file ("/config");

	f = (FILE*) g_fopen (config_file, "w");
	if (f)
	{
		fputs (data, f);
		fclose (f);
	}

	g_free (config_file);

	g_free (data);

	g_key_file_free (file);
}

	gint
load_profile (gchar* name)
{
	GKeyFile*	file;
	GError*		error;
	Profile		profile;
	gint		value_int;
	gboolean	value_bool;
	gchar*		value_str;
	gchar		path [300];
	gchar		text [100];
	gint		i, j;

	/* Get the path */
	if (g_path_is_absolute (name))
		strcpy (path, name);
	else
		strcpy (path, path_from_name (name));

	/* Load the profile */
	file = g_key_file_new ();

	error = NULL;
	if (!g_key_file_load_from_file (file, path, G_KEY_FILE_NONE, &error))
	{
		g_message ("Profile (%s): %s", path, error->message);
		return -1;
	}

	set_default_values (&profile);

	/* Read the values */
#define GET_INT(group,section,key,var) \
	error = NULL; \
	value_int = g_key_file_get_integer (file, group, key, &error); \
	if (error == NULL) profile.section.var = value_int;

#define GET_BOOL(group,section,key,var) \
	error = NULL; \
	value_bool = g_key_file_get_boolean (file, group, key, &error); \
	if (error == NULL) profile.section.var = value_bool;

#define GET_STR(group,section,key,var) \
	error = NULL; \
	value_str = g_key_file_get_string (file, group, key, &error); \
	if (error == NULL) strcpy (profile.section.var, value_str);

	GET_INT ("config", config, "movement_speed", movement_speed);
	GET_INT ("config", config, "auto_calibrate", auto_calibrate);

	GET_INT ("config", config, "menu_time", menu_time);
	GET_INT ("config", config, "keyb_time", keyb_time);

	GET_BOOL ("config", config, "use_switches", use_switches);
	GET_INT ("config", config, "switch_action_0", switch_action [0]);
	GET_INT ("config", config, "switch_action_1", switch_action [1]);
	GET_INT ("config", config, "switch_action_2", switch_action [2]);
	GET_INT ("config", config, "switch_action_3", switch_action [3]);
	GET_INT ("config", config, "long_press_time", long_press_time);

	GET_INT ("config", config, "sensitivity_0", sensitivity [0]);
	GET_INT ("config", config, "sensitivity_1", sensitivity [1]);
	GET_INT ("config", config, "sensitivity_2", sensitivity [2]);
	GET_INT ("config", config, "sensitivity_3", sensitivity [3]);

	GET_INT ("config", config, "menu_size", menu_size);
	GET_INT ("config", config, "menu_use_images", menu_use_images);
	GET_INT ("config", config, "keyb_size", keyb_size);
	GET_INT ("config", config, "keyb_show_dictionary", keyb_show_dictionary);
	GET_INT ("config", config, "keyb_show_layout_bar", keyb_show_layout_bar);

	GET_BOOL ("config", config, "can_disable_menu", can_disable_menu);
	GET_BOOL ("config", config, "auto_click", auto_click);

	GET_BOOL ("main_window", main_window, "show", show);
	GET_INT ("main_window", main_window, "x", x);
	GET_INT ("main_window", main_window, "y", y);
	GET_INT ("main_window", main_window, "width", width);
	GET_INT ("main_window", main_window, "height", height);
	GET_INT ("main_window", main_window, "n_shortcuts", n_shortcuts);

	for (i = 0; i < MAX_SHORTCUTS; i++)
	{
		sprintf (text, "shortcut_title_%d", i);
		GET_STR ("main_window", main_window, text, shortcut_title [i]);

		sprintf (text, "shortcut_uri_%d", i);
		GET_STR ("main_window", main_window, text, shortcut_uri [i]);
	}

	GET_INT ("keyboard", keyboard, "x", x);
	GET_INT ("keyboard", keyboard, "y", y);

	for (i = 0; i < 2; i++)
	{
		for (j = 0; j < 3; j++)
		{
			sprintf (text, "matrix_%d_%d", i, j);
			GET_INT ("calibration", calibration, text, matrix [i][j]);
		}
	}

#undef GET_INT
#undef GET_BOOL
#undef GET_STR

	g_key_file_free (file);

	current_profile = profile;

	/* Remember its name */
	if (g_path_is_absolute (name))
		strcpy (path, name);
	else
		strcpy (current_profile_name, name);

	update_config_file ();

	return 0;
}

	gint
save_current_profile (void)
{
	return save_profile (current_profile_name);
}

	void
create_profile (gchar* name)
{
	save_profile (name);
}

	void
remove_profile (gchar* name)
{
	g_remove (path_from_name (name));
}

	void
rename_profile (gchar* name, gchar* name_new)
{
	gchar path_new [200];

	/* Rename the file */
	strcpy (path_new, path_from_name (name_new));
	g_rename (path_from_name (name), path_new);

	/* And if it is the current profile remember the new name, and
	 * update the config file */
	if (strcmp (name, current_profile_name) == 0)
	{
		strcpy (current_profile_name, name_new);
		update_config_file ();
	}
}

	void
export_profile (gchar* name, gchar* path)
{
	copy_file (path_from_name (name), path);
}

	gboolean
profile_exists (gchar* name)
{
	return g_file_test (path_from_name (name), G_FILE_TEST_EXISTS);
}

	void
send_cfg_speed (void)
{
	/* Menu/clicker */
	if (showing_menu || showing_clicker)
		send_command (COMMAND_CONFIG, CFG_NOTIFY_STEP_TIME, 20);
	/* Movement */
	else
		send_command (COMMAND_CONFIG, CFG_MOVEMENT_SPEED, current_profile.config.movement_speed);
}

	void
send_cfg_time (void)
{
	/* Keyboard */
	if (showing_keyboard && mouse_over_keyboard)
	{
		send_command (COMMAND_CONFIG, CFG_STOP_TIME, 
				(current_profile.config.keyb_time + 1) * 50);
	}
	/* Menu */
	else if (showing_menu)
	{
		send_command (COMMAND_CONFIG, CFG_STOP_TIME, 0);
	}
	/* Clicker */
	else if (showing_clicker)
	{
		send_command (COMMAND_CONFIG, CFG_STOP_TIME,
				(current_profile.config.menu_time + 1) * 50 / CLICKER_N_STEPS);
	}
	/* Movement */
	else
	{
		send_command (COMMAND_CONFIG, CFG_STOP_TIME,
				(current_profile.config.menu_time + 1) * 50);
	}
}

	void
send_cfg_switches (void)
{
	send_command (COMMAND_CONFIG, CFG_LONG_PRESS_TIME,
			(current_profile.config.long_press_time + 1) * 50);
}

	void
send_cfg_sensitivity (void)
{
	gint threshold_left, threshold_right, threshold_up, threshold_down;

	threshold_left	= 12 - (current_profile.config.sensitivity [0] << 1);
	threshold_right	= 12 - (current_profile.config.sensitivity [1] << 1);
	threshold_up	= 12 - (current_profile.config.sensitivity [2] << 1);
	threshold_down	= 12 - (current_profile.config.sensitivity [3] << 1);

	send_command (COMMAND_CONFIG, CFG_THRESHOLD_LEFT,	threshold_left);
	send_command (COMMAND_CONFIG, CFG_THRESHOLD_RIGHT,	threshold_right);
	send_command (COMMAND_CONFIG, CFG_THRESHOLD_UP,		threshold_up);
	send_command (COMMAND_CONFIG, CFG_THRESHOLD_DOWN,	threshold_down);
}

	void
send_cfg_calibration (void)
{
	send_command (COMMAND_CALIB, 0 | (0 << 4), current_profile.calibration.matrix [0][0]);
	send_command (COMMAND_CALIB, 1 | (0 << 4), current_profile.calibration.matrix [1][0]);

	send_command (COMMAND_CALIB, 0 | (1 << 4), current_profile.calibration.matrix [0][1]);
	send_command (COMMAND_CALIB, 1 | (1 << 4), current_profile.calibration.matrix [1][1]);

	send_command (COMMAND_CALIB, 0 | (2 << 4), current_profile.calibration.matrix [0][2]);
	send_command (COMMAND_CALIB, 1 | (2 << 4), current_profile.calibration.matrix [1][2]);
}

	static void
set_default_values (Profile* profile)
{
	gint	i;

	/* Config */
	profile->config.movement_speed = 1;
	profile->config.auto_calibrate = TRUE;

	profile->config.menu_time = 1;
	profile->config.keyb_time = 1;

	profile->config.use_switches = FALSE;
	profile->config.switch_action [0] = CLICK;
	profile->config.switch_action [1] = 0;
	profile->config.switch_action [2] = OPTIONS;
	profile->config.switch_action [3] = 0;
	profile->config.long_press_time = 1;

	profile->config.sensitivity [0] = 3;
	profile->config.sensitivity [1] = 3;
	profile->config.sensitivity [2] = 3;
	profile->config.sensitivity [3] = 3;

	profile->config.menu_size = 0;
	profile->config.menu_use_images = FALSE;
	profile->config.keyb_size = 0;
	profile->config.keyb_show_dictionary = TRUE;
	profile->config.keyb_show_layout_bar = FALSE;

	profile->config.can_disable_menu = FALSE;
	profile->config.auto_click = FALSE;

	/* Main window */
	profile->main_window.show = TRUE;
	profile->main_window.x = G_MAXINT;
	profile->main_window.y = G_MAXINT;
	profile->main_window.width = -1;
	profile->main_window.height = -1;

	profile->main_window.n_shortcuts = 0;

	for (i = 0; i < MAX_SHORTCUTS; i++)
	{
		strcpy (profile->main_window.shortcut_title [i], "");
		strcpy (profile->main_window.shortcut_uri [i], "");
	}

	/* Keyboard */
	profile->keyboard.x = G_MAXINT;
	profile->keyboard.y = G_MAXINT;

	/* Calibration */
	profile->calibration.matrix [0][0] = 127;
	profile->calibration.matrix [0][1] = 0;
	profile->calibration.matrix [0][2] = 0;

	profile->calibration.matrix [1][0] = 0;
	profile->calibration.matrix [1][1] = 127;
	profile->calibration.matrix [1][2] = 0;
};

	static gchar*
path_from_name (gchar* name)
{
	static gchar path [300];
	gchar*	config_file;

	config_file = get_config_file ("/profiles");

	sprintf (path, "%s/%s.enp", config_file, name);

	g_free (config_file);

	return path;
}

	static gint
save_profile (gchar* name)
{
	GKeyFile*	file;
	gchar		text [100];
	gint		i, j;
	gchar*		data;
	FILE*		f;

	/* Create the profile */
	file = g_key_file_new ();

	/* Write the values */
#define SET_INT(group,section,key,var) \
	g_key_file_set_integer (file, group, key, current_profile.section.var);

#define SET_BOOL(group,section,key,var) \
	g_key_file_set_integer (file, group, key, current_profile.section.var);

#define SET_STR(group,section,key,var) \
	g_key_file_set_string (file, group, key, current_profile.section.var);

	SET_INT ("config", config, "movement_speed", movement_speed);
	SET_INT ("config", config, "auto_calibrate", auto_calibrate);

	SET_INT ("config", config, "menu_time", menu_time);
	SET_INT ("config", config, "keyb_time", keyb_time);

	SET_BOOL ("config", config, "use_switches", use_switches);
	SET_INT ("config", config, "switch_action_0", switch_action [0]);
	SET_INT ("config", config, "switch_action_1", switch_action [1]);
	SET_INT ("config", config, "switch_action_2", switch_action [2]);
	SET_INT ("config", config, "switch_action_3", switch_action [3]);
	SET_INT ("config", config, "long_press_time", long_press_time);

	SET_INT ("config", config, "sensitivity_0", sensitivity [0]);
	SET_INT ("config", config, "sensitivity_1", sensitivity [1]);
	SET_INT ("config", config, "sensitivity_2", sensitivity [2]);
	SET_INT ("config", config, "sensitivity_3", sensitivity [3]);

	SET_INT ("config", config, "menu_size", menu_size);
	SET_INT ("config", config, "menu_use_images", menu_use_images);
	SET_INT ("config", config, "keyb_size", keyb_size);
	SET_INT ("config", config, "keyb_show_dictionary", keyb_show_dictionary);
	SET_INT ("config", config, "keyb_show_layout_bar", keyb_show_layout_bar);

	SET_BOOL ("config", config, "can_disable_menu", can_disable_menu);
	SET_BOOL ("config", config, "auto_click", auto_click);

	SET_BOOL ("main_window", main_window, "show", show);
	SET_INT ("main_window", main_window, "x", x);
	SET_INT ("main_window", main_window, "y", y);
	SET_INT ("main_window", main_window, "width", width);
	SET_INT ("main_window", main_window, "height", height);
	SET_INT ("main_window", main_window, "n_shortcuts", n_shortcuts);

	for (i = 0; i < MAX_SHORTCUTS; i++)
	{
		sprintf (text, "shortcut_title_%d", i);
		SET_STR ("main_window", main_window, text, shortcut_title [i]);

		sprintf (text, "shortcut_uri_%d", i);
		SET_STR ("main_window", main_window, text, shortcut_uri [i]);
	}

	SET_INT ("keyboard", keyboard, "x", x);
	SET_INT ("keyboard", keyboard, "y", y);

	for (i = 0; i < 2; i++)
	{
		for (j = 0; j < 3; j++)
		{
			sprintf (text, "matrix_%d_%d", i, j);
			SET_INT ("calibration", calibration, text,	matrix [i][j]);
		}
	}

#undef SET_INT
#undef SET_BOOL
#undef SET_STR

	/* Save the file */
	data = g_key_file_to_data (file, NULL, NULL);

	f = (FILE*) g_fopen (path_from_name (name), "w");
	if (f)
	{
		fputs (data, f);
		fclose (f);
	}

	g_free (data);

	g_key_file_free (file);

	return 0;
}

	static void
get_system_language (void)
{
	const gchar* const* languages;
	gboolean	found;
	gint		i, j;

	/* Check whether one the system's locales is supported */
	languages = g_get_language_names ();

	found = FALSE;
	for (i = 0; languages [i] != NULL; i++)
	{
		for (j = 0; j < N_LANGUAGES; j++)
		{
			if (!g_ascii_strcasecmp (languages [i], language_name [j]))
			{
				current_language = j;
				found = TRUE;
				break;
			}
		}

		if (found)
			break;
	}

	/* Otherwise, use English */
	if (!found)
		current_language = 0;
}

	static void
get_system_keyb_layout (void)
{
	const gchar* const* languages;
	gboolean	found;
	gint		i, j;

	/* Check whether one the system's locales is supported */
	languages = g_get_language_names ();

	found = FALSE;
	for (i = 0; languages [i] != NULL; i++)
	{
		for (j = 0; j < N_KEYB_LAYOUTS; j++)
		{
			if (!g_ascii_strcasecmp (languages [i], keyb_layout_name [j]))
			{
				current_keyb_layout = j;
				found = TRUE;
				break;
			}
		}

		if (found)
			break;
	}

	/* Otherwise, use en_US */
	if (!found)
		current_keyb_layout = 0;
}
